#' Retrieves quarterly master index.
#'
#' \code{getMasterIndex} retrieves the quarterly master index from US SEC site.
#'
#' getMasterIndex function takes filing year as an input parameter from user,  
#' download quarterly master index from ftp://ftp.sec.gov/edgar/full-index.
#' It strips the headers, converts into data frame, and merges such quarterly
#' data frames into yearly data frames and stored it in Rda format.
#' Function creates new directory 'Master Index' into working directory 
#' to save these Rda Master Index. Please note, for all other functions in this 
#' package needs to locate the same working directory to access these Rda master index files.  
#'  
#' @usage getMasterIndex(year.array)
#'
#' @param year.array year in integer or integer array containing years for which master 
#' index are to be downloaded.
#' 
#' @return Function retrieves quarterly master index files 
#' from \url{ftp://ftp.sec.gov/edgar/full-index} site and returns download status data frame.
#'   
#' @examples
#' \dontrun{
#' 
#' report <- getMasterIndex(1995) 
#' ## Download quarterly master index files for the year 1990 and stores into yearly  
#' ## 1995master.Rda file. It returns download report in data frame format.
#' 
#' report <- getMasterIndex(c(1994, 1995, 2006)) 
#' ## Download quarterly master index files for the years 1994, 1995, 2006 and stores into 
#' ## different {year}master.Rda files. It returns download report in data frame format.
#'}

getMasterIndex <- function(year.array) {
  if (!is.numeric(year.array)) {
    cat("Please provide valid year.")
    return()
  }
  
  # function to download file and return FALSE if download
  # error
  DownloadFile <- function(link, dfile) {
    tryCatch({
      utils::download.file(link, dfile, quiet = TRUE)
      return(TRUE)
    }, error = function(e) {
      return(FALSE)
    })
  }
  
  options(warn = -1)
  dir.create("Master Index")
  
  status.array <- data.frame()
  
  for (i in 1:length(year.array)) {
    year <- year.array[i]
    year.master <- data.frame()
    quarterloop <- 4
    
    # Find the number of quarter completed in that year
    if (year == format(Sys.Date(), "%Y")) {
      quarterloop <- ceiling(as.integer(format(Sys.Date(), 
        "%m"))/3)
    }
    
    for (quarter in 1:quarterloop) {
      # save downloaded file as specific name
      dfile <- paste0("Master Index/", year, "QTR", quarter, 
        "master.gz")
      file <- paste0("Master Index/", year, "QTR", quarter, 
        "master")
      
      # form a link to download master file
      link <- paste0("ftp://ftp.sec.gov/edgar/full-index/", 
        year, "/QTR", quarter, "/master.gz")
      
      res <- DownloadFile(link, dfile)
      if (res) {
        # Unzip gz file
        R.utils::gunzip(dfile, destname = file, temporary = FALSE, 
          skip = FALSE, overwrite = TRUE, remove = TRUE)
        cat("Successfully downloaded Master Index for year:", 
          year, "and quarter:", quarter, "\n")
        
        # Removing ''' so that scan with '|' not fail due to
        # occurrence of ''' in company name
        data <- gsub("'", "", readLines(file))
        # writting back to storage
        writeLines(data, file)
        
        d <- scan(file, what = list("", "", "", "", ""), 
          flush = F, skip = 10, sep = "|", quiet = T)
        
        # Remove punctuation characters from company names
        COMPANY_NAME <- gsub("[[:punct:]]", " ", d[[2]], 
          perl = T)
        
        data <- data.frame(CIK = d[[1]], COMPANY_NAME = COMPANY_NAME, 
          FORM_TYPE = d[[3]], DATE_FILED = d[[4]], EDGAR_LINK = d[[5]], 
          QUARTER = quarter)
        year.master <- rbind(year.master, data)
        file.remove(file)
        status.array <- rbind(status.array, data.frame(Filename = paste0(year, 
          ": quarter-", quarter), status = "Download success"))
      } else {
        status.array <- rbind(status.array, data.frame(Filename = paste0(year, 
          ": quarter-", quarter), status = "Server Error"))
      }
    }
    
    assign(paste0(year, "master"), year.master)
    save(year.master, file = paste0("Master Index/", year, 
      "master.Rda"))
  }
  return(status.array)
}
